'
'                       QBasic Sprite Editor v0.21
'               
'                                   By
'
'                    David Tordrup of The QBasic Team
'
'
' Features
'
' * Quick and easy loading and saving
' * Tools for line, boxes, ellipses, circles and filling
' * Sprites up to 115 x 100 pixels in size
' * 16 color palette
' * Auto-truncate blank areas of sprite on save
' * Context sensitive help
'
'
DECLARE SUB Help ()
DECLARE SUB LoadSprite ()
DECLARE SUB Message (Row%, Col%, Text$, FgColor%, BgColor%, VerSpace%, HorSpace%, Pause%)
DECLARE SUB ChangeName ()
DECLARE SUB CalcSize ()
DECLARE SUB DrawIcons ()
DECLARE SUB ChangeTool ()
DECLARE SUB FillArea ()
DECLARE SUB SaveSprite ()
DECLARE SUB ChangeColor ()
DECLARE SUB DrawBox ()
DECLARE SUB DrawCircle ()
DECLARE SUB MainLoop ()
DECLARE FUNCTION GetChar$ ()
DECLARE SUB InitVars ()
DECLARE SUB Intro ()
DECLARE SUB InitEditor ()
DECLARE SUB box (X1%, y1%, X2%, y2%, Clr%)
DECLARE SUB ColorButtons ()
DECLARE SUB DrawWin (X1%, y1%, X2%, y2%, BgColor%, Status%, Density%)
DECLARE SUB Center (Row%, Text$, FgColor%, BgColor%)
DECLARE SUB KeyPause ()
DECLARE SUB DrawLine ()

ON ERROR GOTO ErrorHandler

TYPE SpriteType
  Size AS INTEGER
  X1 AS INTEGER
  y1 AS INTEGER
  X2 AS INTEGER
  y2 AS INTEGER
END TYPE

TYPE ControlType
  FileName AS STRING * 12
  UseColor AS INTEGER
  UseTool AS INTEGER
  MarkerX AS INTEGER
  MarkerY AS INTEGER
END TYPE

DIM SHARED Control AS ControlType
DIM SHARED Sprite AS SpriteType
DIM SHARED Marker%(3)
DIM SHARED Icons%(550)

SCREEN 7

InitVars
Intro

InitEditor
MainLoop

SYSTEM

FunctionKeys:
DATA 6
DATA 8, 24, 14, "F2 - ", 15, "Color"
DATA 9, 24, 14, "F3 - ", 15, "Tools"
DATA 10, 24, 14, "F4 - ", 15, "Filename"
DATA 11, 24, 14, "F5 - ", 15, "Save"
DATA 12, 24, 14, "F6 - ", 15, "Load"
DATA 14, 24, 12, "F10- ", 15, "Quit"


MainHelp:
DATA 19
DATA 14, "Use the arrow keys to navigate"
DATA 14, "the cursor, and press SPACE or"
DATA 14, "ENTER to activate your current"
DATA 14, "tool."
DATA 14, ""
DATA 12, "Pressing F2 and F3 lets you change"
DATA 12, "the color and tool used. Use the"
DATA 12, "arrow keys to select the desired
DATA 12, "option and press ENTER."
DATA 12, ""
DATA 14, "You can change the filename for"
DATA 14, "your sprite by pressing F4, and"
DATA 14, "typing in a valid DOS filename."
DATA 14, ""
DATA 12, "Press F5 to save your work in the"
DATA 12, "designated file, or F6 to load a"
DATA 12, "previously created file."
DATA 12, ""
DATA 11, "         Press any key..."

ChangeColorHelp:
DATA 9
DATA 14, "Use the arrow keys to select the"
DATA 14, "desired color, and press ENTER"
DATA 14, "to activate it."
DATA 14, ""
DATA 12, "NOTE: Pixels drawn in black are"
DATA 12, "considered void unless drawn on"
DATA 12, "a colored background."
DATA 12, ""
DATA 11, "         Press any key..."


ChangeToolHelp:
DATA 17
DATA 14, "Use the arrow keys to select the"
DATA 14, "desired tool, and press ENTER"
DATA 14, "to activate it."
DATA 14, ""
DATA 12, "Freehand: Lets you plot pixels one"
DATA 12, "          by one."
DATA 12, "Line:     Draws a straight line."
DATA 12, "Box:      Draws an empty box."
DATA 12, "Circle:   Draws a circle or"
DATA 12, "          ellipse. Use the Up/Down"
DATA 12, "          arrows to change the"
DATA 12, "          size of the circle, and"
DATA 12, "          Left/Right to change the"
DATA 12, "          height and width ratio."
DATA 12, "Fill:     Fills an enclosed area."
DATA 12, ""
DATA 11, "         Press any key..."

NotReadable:
DATA 8
DATA 14, "The file specified is not a QBSE"
DATA 14, "readable file. This means that "
DATA 14, "either the file has some header"
DATA 14, "information that is not used by"
DATA 14, "QBSE, or the file is meant for"
DATA 14, "a different screen mode than EGA."
DATA 14, ""
DATA 11, "         Press any key..."

FileNotFound:
DATA 11
DATA 12, "Make sure you have typed the file-"
DATA 12, "name correctly!"
DATA 12, ""
DATA 14, "The file specified is not present"
DATA 14, "within the current directory. You"
DATA 14, "must keep your sprite files in"
DATA 14, "the same directory as this program"
DATA 14, "or you will not be able to load"
DATA 14, "them into the editor."
DATA 14, ""
DATA 11, "         Press any key..."

BadFileName:
DATA 11
DATA 14, "The specified filename contains"
DATA 14, "invalid characters and can not"
DATA 14, "be used to store your sprite."
DATA 14, "Type in a new filename and try"
DATA 14, "saving again."
DATA 14, ""
DATA 12, "Consult your DOS manual for"
DATA 12, "information on valid DOS file-"
DATA 12, "names."
DATA 12, ""
DATA 11, "         Press any key..."

TooLarge:
DATA 10
DATA 14, "The file you are trying to load"
DATA 14, "is larger than 115x100 pixels, "
DATA 14, "and can therefore not be loaded"
DATA 14, "into the editor."
DATA 14, ""
DATA 12, "This error may also occur if you"
DATA 12, "are trying to load sprites from"
DATA 12, "other screen modes than 7."
DATA 12, ""
DATA 11, "         Press any key..."

ErrorHandler:
IF ERR = 54 THEN
  Message 12, 10, "Not a QBSE readable file", 15, 0, 8, 8, 0
  a$ = GetChar$
  IF a$ = CHR$(0) + ";" THEN
    RESTORE NotReadable
    Help
  END IF
  SCREEN 7, , 0, 0
  RESUME RecoverFileError
ELSEIF ERR = 53 THEN
  Message 10, 12, "File not found", 12, 0, 8, 8, 0
  a$ = GetChar$
  IF a$ = CHR$(0) + ";" THEN
    RESTORE FileNotFound
    Help
  END IF
  SCREEN 7, , 0, 0
  RESUME RecoverFileError
ELSEIF ERR = 52 OR ERR = 64 THEN
  Message 10, 12, "Bad File Name!", 15, 0, 8, 8, 0
  a$ = GetChar$
  IF a$ = CHR$(0) + ";" THEN
    RESTORE BadFileName
    Help
  END IF
  SCREEN 7, , 0, 0
  RESUME RecoverFileError
ELSEIF ERR = 5 THEN
  Message 10, 6, "Sprite exceeds maximum size", 14, 0, 8, 8, 0
  a$ = GetChar$
  IF a$ = CHR$(0) + ";" THEN
    RESTORE TooLarge
    Help
  END IF
  SCREEN 7, , 0, 0
  ELSE
  SCREEN 0, 0, 0, 0
  WIDTH 80
  COLOR 7, 0
  CLS
  PRINT "You have encountered an unexpected error. Please report the following"
  PRINT "error code to The QBasic Team:"
  PRINT "E"; ERR; "L"; ERL
  SYSTEM
END IF

RecoverFileError:
MainLoop

SUB box (X1%, y1%, X2%, y2%, Clr%)

IF Clr% = 0 THEN COLOR , 0

LINE (X1%, y1%)-(X2%, y2%), Clr%, BF

END SUB

SUB CalcSize

' Calculate size of sprite in bytes

Sprite.X1 = 145
Sprite.X2 = 30
Sprite.y1 = 150
Sprite.y2 = 50

FOR Col% = 50 TO 150
  FOR Row% = 30 TO 145
     IF POINT(Row%, Col%) THEN
       IF Row% < Sprite.X1 THEN Sprite.X1 = Row%
       IF Col% < Sprite.y1 THEN Sprite.y1 = Col%
       IF Col% > Sprite.y2 THEN Sprite.y2 = Col%
       IF Row% > Sprite.X2 THEN Sprite.X2 = Row%
     END IF
  NEXT Row%
NEXT Col%

Sprite.Size = (4 + (INT(((Sprite.X2 - Sprite.X1 + 1) + 7) / 8) * 4 * (Sprite.y2 - Sprite.y1 + 1)))

END SUB

SUB Center (Row%, Text$, FgColor%, BgColor%)

COLOR FgColor%, BgColor%
LOCATE Row%, 20 - (LEN(Text$) / 2) + 1
PRINT Text$

END SUB

SUB ChangeColor

DO
  SELECT CASE GetChar$
    CASE CHR$(0) + ";"
      RESTORE ChangeColorHelp
      Help
    CASE CHR$(0) + "H"
      IF Control.UseColor > 7 THEN Control.UseColor = Control.UseColor - 8
    CASE CHR$(0) + "P"
      IF Control.UseColor < 8 THEN Control.UseColor = Control.UseColor + 8
    CASE CHR$(0) + "K"
      IF Control.UseColor > 0 THEN Control.UseColor = Control.UseColor - 1
    CASE CHR$(0) + "M"
      IF Control.UseColor < 15 THEN Control.UseColor = Control.UseColor + 1
    CASE CHR$(13), " "
      EXIT DO
    CASE ELSE
  END SELECT
  ColorButtons
LOOP

END SUB

SUB ChangeName

StringName$ = ""
COLOR 15
LOCATE 4, 5: PRINT STRING$(8, " ") + ".   "

DO WHILE Done% = 0
a$ = INKEY$
IF a$ <> "" THEN
   IF a$ = CHR$(8) THEN
     IF LEN(StringName$) > 0 AND LEN(StringName$) < 9 OR LEN(StringName$) > 9 THEN
       StringName$ = LEFT$(StringName$, LEN(StringName$) - 1)
       LOCATE 4, 5: PRINT StringName$ + " "
     ELSEIF LEN(StringName$) = 9 THEN
       StringName$ = LEFT$(StringName$, 7)
       LOCATE 4, 5: PRINT StringName$ + " "
     END IF
   ELSEIF a$ = CHR$(9) THEN
     StringName$ = StringName$
   ELSEIF a$ = CHR$(13) THEN
     Done% = 1
   ELSEIF LEFT$(a$, 1) = CHR$(0) THEN
     a$ = a$
   ELSEIF a$ = "." THEN
     IF LEN(StringName$) < 8 THEN
       StringName$ = StringName$ + SPACE$(8 - LEN(StringName$)) + "."
       LOCATE 4, 5: PRINT StringName$ + " "
     ELSE
     END IF
   ELSEIF a$ <> " " THEN
     IF LEN(StringName$) < 12 THEN
       StringName$ = StringName$ + UCASE$(a$)
       LOCATE 4, 5: PRINT StringName$
     END IF
   END IF
END IF
IF LEN(StringName$) = 8 THEN StringName$ = StringName$ + "."
LOOP

OPEN StringName$ FOR BINARY AS #1
CLOSE #1

IF LEN(StringName$) < 8 THEN
  StringName$ = StringName$ + SPACE$(8 - LEN(StringName$)) + ".GFX"
ELSEIF LEN(StringName$) = 9 THEN
  StringName$ = StringName$ + "GFX"
END IF

Control.FileName = StringName$
LOCATE 4, 5
PRINT Control.FileName

END SUB

SUB ChangeTool

DO
  SELECT CASE GetChar$
    CASE CHR$(0) + ";"
      RESTORE ChangeToolHelp
      Help
    CASE CHR$(0) + "H"
      IF Control.UseTool > 1 THEN Control.UseTool = Control.UseTool - 1
    CASE CHR$(0) + "P"
      IF Control.UseTool < 5 THEN Control.UseTool = Control.UseTool + 1
    CASE CHR$(13)
      EXIT DO
    CASE ELSE
  END SELECT
 DrawIcons
LOOP

END SUB

SUB ColorButtons

' Draw the selecting-buttons
' Selected color is depressed.

FOR a% = 30 TO 135 STEP 15
  IF Control.UseColor = ((a% / 15) - 2) THEN Status% = 0 ELSE Status% = 1
  DrawWin a%, 160, a% + 10, 170, (a% / 15) - 2, Status%, 0
NEXT a%

FOR a% = 30 TO 135 STEP 15
  IF Control.UseColor = ((a% / 15) + 6) THEN Status% = 0 ELSE Status% = 1
  DrawWin a%, 175, a% + 10, 185, (a% / 15) + 6, Status%, 0
NEXT a%
END SUB

SUB DrawBox

' Origin of line
XOld = Control.MarkerX
YOld = Control.MarkerY

PCOPY 0, 1
SCREEN 7, , 1, 1

DO

  SELECT CASE GetChar$

    CASE CHR$(0) + "H"
      PCOPY 0, 1
      IF Control.MarkerY > 0 THEN Control.MarkerY = Control.MarkerY - 1

    CASE CHR$(0) + "P"
      PCOPY 0, 1
      IF Control.MarkerY < 100 THEN Control.MarkerY = Control.MarkerY + 1

    CASE CHR$(0) + "K"
      PCOPY 0, 1
      IF Control.MarkerX > 0 THEN Control.MarkerX = Control.MarkerX - 1

    CASE CHR$(0) + "M"
      PCOPY 0, 1
      IF Control.MarkerX < 115 THEN Control.MarkerX = Control.MarkerX + 1

    CASE CHR$(13)
      PCOPY 1, 0
      EXIT DO
  CASE ELSE
  END SELECT

  LINE (30 + XOld, 50 + YOld)-(30 + Control.MarkerX, 50 + Control.MarkerY), Control.UseColor, B

LOOP

SCREEN 7, , 0, 0
END SUB

SUB DrawCircle

PCOPY 0, 1
SCREEN 7, , 1, 1
Ratio! = 1

DO

  SELECT CASE GetChar$

    CASE CHR$(0) + "H"
      PCOPY 0, 1
      IF Radius% + Control.MarkerX < 115 THEN
        IF Control.MarkerX - Radius% > 0 THEN
          IF Radius% + Control.MarkerY < 100 THEN
            IF Control.MarkerY - Radius% > 0 THEN
              Radius% = Radius% + 1
            END IF
          END IF
        END IF
      END IF
    CASE CHR$(0) + "P"
      PCOPY 0, 1
      IF Radius% > 0 THEN Radius% = Radius% - 1

    CASE CHR$(0) + "K"
      PCOPY 0, 1
      Ratio! = Ratio! + .1

    CASE CHR$(0) + "M"
      PCOPY 0, 1
      Ratio! = Ratio! - .1
  
    CASE CHR$(13)
      PCOPY 1, 0
      EXIT DO
  CASE ELSE
  END SELECT

  CIRCLE (30 + Control.MarkerX, 50 + Control.MarkerY), Radius%, Control.UseColor, , , Ratio!
LOOP



END SUB

SUB DrawIcons

IF Control.UseTool = 1 THEN Status% = 1 ELSE Status% = 0
DrawWin 155, 49, 170, 63, 0, Status%, 1
PUT STEP(-13, -12), Icons%(0)

IF Control.UseTool = 2 THEN Status% = 1 ELSE Status% = 0
DrawWin 155, 71, 170, 85, 0, Status%, 1
PUT STEP(-13, -12), Icons%(110)
           
IF Control.UseTool = 3 THEN Status% = 1 ELSE Status% = 0
DrawWin 155, 93, 170, 107, 0, Status%, 1
PUT STEP(-13, -12), Icons%(220)

IF Control.UseTool = 4 THEN Status% = 1 ELSE Status% = 0
DrawWin 155, 115, 170, 129, 0, Status%, 1
PUT STEP(-13, -12), Icons%(330)

IF Control.UseTool = 5 THEN Status% = 1 ELSE Status% = 0
DrawWin 155, 137, 170, 151, 0, Status%, 1
PUT STEP(-13, -12), Icons%(440)

END SUB

SUB DrawLine

' Origin of line
XOld = Control.MarkerX
YOld = Control.MarkerY

PCOPY 0, 1
SCREEN 7, , 1, 1

DO
 
  SELECT CASE GetChar$
 
    CASE CHR$(0) + "H"
      PCOPY 0, 1
      IF Control.MarkerY > 0 THEN Control.MarkerY = Control.MarkerY - 1
 
    CASE CHR$(0) + "P"
      PCOPY 0, 1
      IF Control.MarkerY < 100 THEN Control.MarkerY = Control.MarkerY + 1

    CASE CHR$(0) + "K"
      PCOPY 0, 1
      IF Control.MarkerX > 0 THEN Control.MarkerX = Control.MarkerX - 1

    CASE CHR$(0) + "M"
      PCOPY 0, 1
      IF Control.MarkerX < 115 THEN Control.MarkerX = Control.MarkerX + 1

    CASE CHR$(13)
      PCOPY 1, 0
      EXIT DO
  CASE ELSE
  END SELECT

  LINE (30 + XOld, 50 + YOld)-(30 + Control.MarkerX, 50 + Control.MarkerY), Control.UseColor
LOOP

SCREEN 7, , 0, 0

END SUB

SUB DrawWin (X1%, y1%, X2%, y2%, BgColor%, Status%, Density%)

IF BgColor% = 0 THEN COLOR , 0

Shade% = 8
IF Density% = 1 THEN Light% = 15 ELSE Light% = 7

IF Status% = 0 THEN
   Col1% = Shade%
   Col2% = Light%
ELSE
  Col1% = Light%
  Col2% = Shade%
END IF
   

LINE (X1%, y1%)-(X1%, y2%), Col1%
LINE (X1%, y1%)-(X2%, y1%), Col1%
LINE (X1% + 1, y2%)-(X2%, y2%), Col2%
LINE (X2%, y1% + 1)-(X2%, y2%), Col2%
LINE (X1% + 1, y1% + 1)-(X2% - 1, y2% - 1), BgColor%, BF

END SUB

SUB FillArea

PCOPY 0, 1
SCREEN 7, , 1, 0
PAINT (30 + Control.MarkerX, 50 + Control.MarkerY), Control.UseColor

IF POINT(29, 49) = 8 THEN PCOPY 1, 0

SCREEN 7, , 0, 0


END SUB

FUNCTION GetChar$

DO
a$ = INKEY$

LOOP UNTIL a$ <> ""
GetChar$ = a$

END FUNCTION

SUB Help

PCOPY 0, 1
SCREEN 7, , 1, 1

DrawWin 20, 20, 300, 185, 0, 0, 0

READ Lines%
FOR a% = 1 TO Lines%
  READ Clr%, Text$
  COLOR Clr%
  LOCATE 3 + a%, 4
  PRINT Text$
NEXT a%

KeyPause

SCREEN 7, , 0, 0

END SUB

SUB InitEditor

' Editing window
DrawWin 29, 49, 146, 151, 0, 0, 0

' Draw preview window
'DrawWin 160, 50, 190, 80, 0, 0   ,0

' Draw color selecting buttons
ColorButtons

' Draw tool icons
DrawIcons

' Print basic help
DrawWin 179, 49, 290, 120, 0, 1, 0

RESTORE FunctionKeys
READ NumKeys%
FOR a% = 1 TO NumKeys%
  READ Row%, Col%, ClrK%, Key$, ClrF%, Function$
  LOCATE Row%, Col%
  COLOR ClrK%
  PRINT Key$;
  COLOR ClrF%
  PRINT Function$
NEXT a%

' Size-of-sprite box
DrawWin 179, 130, 290, 151, 0, 1, 0

' Filename box
DrawWin 29, 20, 146, 35, 0, 1, 0

' Display help notice
DrawWin 179, 20, 290, 35, 0, 1, 0
COLOR 14
LOCATE 4, 24
PRINT "F1 - "; : COLOR 15: PRINT "Help"

END SUB

SUB InitVars

Control.UseColor = 7
Control.UseTool = 1

' Draw cursor for editing
PSET (1, 1), 15
GET (1, 1)-(1, 1), Marker%

Control.FileName = "NONAME00.GFX"

DEF SEG = VARSEG(Icons%(0))
BLOAD "freehand.gfx", 0
BLOAD "line.gfx", VARPTR(Icons%(110))
BLOAD "box.gfx", VARPTR(Icons%(220))
BLOAD "circle.gfx", VARPTR(Icons%(330))
BLOAD "fill.gfx", VARPTR(Icons%(440))

DEF SEG

END SUB

SUB Intro

DrawWin 0, 0, 319, 199, 1, 0, 0
DrawWin 1, 1, 318, 198, 1, 0, 0

Center 5, "The QBasic Team presents:", 15, 1

Center 9, "QBasic Sprite Editor v0.21", 12, 1

Center 22, "Copyright (c) 1997 by David Tordrup", 14, 1
DrawWin 90, 105, 230, 125, 1, 1, 0
Center 15, "Press any key...", 7, 1
KeyPause

' Redraw screen border and clear screen
DrawWin 0, 0, 319, 199, 1, 1, 0
DrawWin 1, 1, 318, 198, 1, 1, 0


END SUB

SUB KeyPause

DO
LOOP UNTIL INKEY$ <> ""

END SUB

SUB LoadSprite

PCOPY 0, 1

SCREEN 7, , 1, 1

DrawWin 90, 67, 208, 104, 0, 1, 0
LOCATE 10, 13
COLOR 15
PRINT "Filename: "

StringName$ = ""
COLOR 15
LOCATE 12, 13: PRINT STRING$(8, " ") + ".   "

DO WHILE Done% = 0
a$ = INKEY$
IF a$ <> "" THEN
   IF a$ = CHR$(8) THEN
     IF LEN(StringName$) > 0 AND LEN(StringName$) < 9 OR LEN(StringName$) > 9 THEN
       StringName$ = LEFT$(StringName$, LEN(StringName$) - 1)
       LOCATE 12, 13: PRINT StringName$ + " "
     ELSEIF LEN(StringName$) = 9 THEN
       StringName$ = LEFT$(StringName$, 7)
       LOCATE 12, 13: PRINT StringName$ + " "
     END IF
   ELSEIF a$ = CHR$(9) THEN
     StringName$ = StringName$
   ELSEIF a$ = CHR$(13) THEN
     Done% = 1
   ELSEIF LEFT$(a$, 1) = CHR$(0) THEN
     a$ = a$
   ELSEIF a$ = "." THEN
     IF LEN(StringName$) < 8 THEN
       StringName$ = StringName$ + SPACE$(8 - LEN(StringName$)) + "."
     ELSE
     END IF
   ELSEIF a$ <> " " THEN
     IF LEN(StringName$) < 12 THEN
       StringName$ = StringName$ + UCASE$(a$)
       LOCATE 12, 13: PRINT StringName$
     END IF
   END IF
END IF
IF LEN(StringName$) = 8 THEN StringName$ = StringName$ + "."
LOOP

SCREEN 7, , 0, 0

DIM Loaded%(20000)

DEF SEG = VARSEG(Loaded%(0))
BLOAD StringName$, 0

CalcSize
Control.FileName = StringName$
SCREEN 7, , 0, 0
InitEditor
PUT (30 + (115 / 2 - Loaded%(0) / 2), 50 + (100 / 2 - Loaded%(1) / 2)), Loaded%

ERASE Loaded%

END SUB

SUB MainLoop

DO

  PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
 
  COLOR 15
  LOCATE 18, 24: PRINT "Size:"; :
  COLOR 14
  PRINT USING "#####"; Sprite.Size
 
  COLOR 15
  LOCATE 4, 5
  PRINT Control.FileName
 
  SELECT CASE GetChar$
    CASE CHR$(0) + "H"
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
      IF Control.MarkerY > 0 THEN Control.MarkerY = Control.MarkerY - 1

    CASE CHR$(0) + "P"
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
      IF Control.MarkerY < 100 THEN Control.MarkerY = Control.MarkerY + 1

    CASE CHR$(0) + "K"
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
      IF Control.MarkerX > 0 THEN Control.MarkerX = Control.MarkerX - 1

    CASE CHR$(0) + "M"
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
      IF Control.MarkerX < 115 THEN Control.MarkerX = Control.MarkerX + 1

    CASE " ", CHR$(13)
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
      Action% = Control.UseTool
      IF Action% = 1 THEN
         PSET (30 + Control.MarkerX, 50 + Control.MarkerY), Control.UseColor
      ELSEIF Action% = 2 THEN
         DrawLine
      ELSEIF Action% = 3 THEN
         DrawBox
      ELSEIF Action% = 4 THEN
         DrawCircle
      ELSEIF Action% = 5 THEN
         FillArea
      END IF
   
    CASE CHR$(0) + ";"
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
      RESTORE MainHelp
      Help

    CASE CHR$(0) + "<"
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
      ChangeColor
 
    CASE CHR$(0) + "="
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
      ChangeTool
   
    CASE CHR$(0) + ">"
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
      ChangeName

    CASE CHR$(0) + "?"
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
      SaveSprite
   
    CASE CHR$(0) + "@"
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
      LoadSprite

    CASE CHR$(0) + "D"
      EXIT DO

    CASE ELSE
      PUT (30 + Control.MarkerX, 50 + Control.MarkerY), Marker%, XOR
 
  END SELECT
       
LOOP
END SUB

SUB Message (Row%, Col%, Text$, FgColor%, BgColor%, VerSpace%, HorSpace%, Pause%)

PCOPY 0, 2
SCREEN 7, , 2, 2

X1% = Col% * 8 - (8 + HorSpace%)
y1% = Row% * 8 - (8 + VerSpace%)

X2% = X1% + LEN(Text$) * 8 + HorSpace% * 2
y2% = y1% + 8 + VerSpace% * 2

DrawWin X1%, y1%, X2%, y2%, 0, BgColor%, 0

LOCATE Row%, Col%
COLOR FgColor%

PRINT Text$

IF Pause% THEN
  KeyPause
  SCREEN 7, , 0, 0
ELSE
END IF

END SUB

SUB PrintHelp

' Print help on the screen
DrawWin 160, 20, 300, 140, 0, 1, 0
LOCATE 4, 22
COLOR 15
PRINT "keys:"
COLOR 14
LOCATE 7, 22
PRINT "Arrow keys -"
LOCATE 8, 23
PRINT "Move cursor"
LOCATE 10, 22
PRINT "Space bar -"
LOCATE 11, 23
PRINT "Plot pixel"
LOCATE 13, 22
PRINT "F1 - Help"
LOCATE 14, 22
PRINT "F2 - Color"
LOCATE 15, 22
PRINT "F3 - Filename"
LOCATE 16, 22
PRINT "F4 - Save"

END SUB

SUB SaveSprite

CalcSize

IF Sprite.X1 = 145 AND Sprite.X2 = 30 THEN
  
  Message 10, 7, "Can't save a blank picture", 14, 0, 8, 8, 1
  Sprite.Size = 0

ELSE

  DIM Sprites%(Sprite.Size)
  GET (Sprite.X1, Sprite.y1)-(Sprite.X2, Sprite.y2), Sprites%
  DEF SEG = VARSEG(Sprites%(0))
  BSAVE Control.FileName, VARPTR(Sprites%(0)), Sprite.Size
  SCREEN 7, , 0, 0

END IF
END SUB

